==> tfd-src-0.1.tgz: tfd-src-0.1/search/domain_transition_graph.h <==
#ifndef DOMAIN_TRANSITION_GRAPH_H
#define DOMAIN_TRANSITION_GRAPH_H

#include <iostream>
#include <map>
#include <vector>
#include <tr1/unordered_map>
//#include "globals.h"
#include "operator.h"
using namespace std;

class CGHeuristic;
class TimeStampedState;
class Operator;

class ValueNode;
class ValueTransition;
class ValueTransitionLabel;
class DomainTransitionGraph;

// Note: We do not use references but pointers to refer to the "parents" of
// transitions and nodes. This is because these structures could not be
// put into vectors otherwise.

typedef multimap<int, ValueNode *> Heap;
typedef std::tr1::unordered_map<int, int> hashmap; 
typedef hashmap::value_type ValuePair;

struct PrevailCondition {
    DomainTransitionGraph *prev_dtg;
    int local_var;
    double value;
    enum var_type {discrete_var = 0, numerical_var = 1} type;
    PrevailCondition(DomainTransitionGraph *prev, int var, double val,
	    var_type the_type) :
	prev_dtg(prev), local_var(var), value(val), type(the_type) {
    }
    PrevailCondition(istream &in);
    void dump() const;
};

struct ValueTransitionLabel {
    int duration_variable;
    Operator *op;
    vector<PrevailCondition> all_prevails;
    ValueTransitionLabel(int the_duration_variable, Operator *theOp,
	    const vector<PrevailCondition> &the_prevails) :
	duration_variable(the_duration_variable), op(theOp),
		all_prevails(the_prevails) {
    }

    //for goals
    ValueTransitionLabel(int the_duration_variable,
	    const vector<PrevailCondition> &the_prevails) :
	duration_variable(the_duration_variable), all_prevails(the_prevails) {
	op = NULL;
    }
    void dump() const;
};

struct ValueTransition {
    ValueNode *target;
    vector<ValueTransitionLabel> labels;
    vector<ValueTransitionLabel> ccg_labels; // labels for cyclic CG heuristic

    ValueTransition(ValueNode *targ) :
	target(targ) {
    }
    void dump() const;
    void simplify();
};

struct ValueNode {
    DomainTransitionGraph *parent_graph;
    int value;
    vector<ValueTransition> transitions;

    vector<int> distances; // cg; empty vector if not yet requested
    vector<ValueTransitionLabel *> helpful_transitions;
    // cg; empty vector if not requested
    Heap::iterator pos_in_heap; // cg
    vector<double> children_state; // cg
    ValueNode *reached_from; // cg
    //ValueNode *helpful_reached_from;     // cg
    ValueTransitionLabel *reached_by; // cg
    //ValueTransitionLabel *helpful_reached_by; // cg

    ValueNode(DomainTransitionGraph *parent, int val) :
	parent_graph(parent), value(val), reached_from(0), reached_by(0) {
    }
    void dump() const;
};

struct CompTransition {
    int left_var;
    int right_var;
    binary_op op;

    void dump() const;
};

struct FuncTransition {
    int starting_variable;
    vector<PrevailCondition> all_prevails;
    assignment_op a_op;
    int influencing_variable;
    int duration_variable;
    Operator *op;
    FuncTransition(int the_starting_variable,
	    vector<PrevailCondition> the_prevails, assignment_op the_a_op,
	    int the_influencing_variable, int the_duration_variable,
	    Operator *the_op) :
	starting_variable(the_starting_variable), all_prevails(the_prevails),
		a_op(the_a_op), influencing_variable(the_influencing_variable),
		duration_variable(the_duration_variable), op(the_op) {
    }
};

class DomainTransitionGraph {
    friend class CGHeuristic;
    friend class ValueNode;
    friend class ValueTransition;
    friend class Transition;

public:
    int var;
    bool is_axiom;

    int last_helpful_transition_extraction_time; // cg heuristic; "dirty bit"

    vector<int> local_to_global_child;
    // used for mapping variables in conditions to their global index 
    // (only needed for initializing child_state for the start node?)
    vector<int> ccg_parents;
    // Same as local_to_global_child, but for cyclic CG heuristic.
    hashmap global_to_local_ccg_parents;

public:

    virtual ~DomainTransitionGraph() {
    }
    static void read_all(istream &in);
    virtual void read_data(istream &in) = 0;
    static void compute_causal_graph_parents_comp(int var, map<int, int> &global_to_ccg_parent);
    static void collect_func_transitions(int var, map<int, int> &global_to_ccg_parent);
    virtual void dump() const = 0;
};

class DomainTransitionGraphSymb : public DomainTransitionGraph {
    friend class CGHeuristic;
    friend class ValueNode;
    friend class ValueTransition;
    friend class Transition;
public:
    vector<ValueNode> nodes;

    DomainTransitionGraphSymb(const DomainTransitionGraphSymb &other); // copying forbidden

public:
    DomainTransitionGraphSymb(int var_index, int node_count);
    virtual void read_data(istream &in);
    virtual void dump() const;
    virtual void get_successors(int value, vector<int> &result) const;
    // Build vector of values v' such that there is a transition from value to v'.
};

class DomainTransitionGraphSubterm : public DomainTransitionGraph {
    friend class CGHeuristic;
    friend class ValueNode;
    friend class ValueTransition;
    friend class Transition;

    int left_var;
    int right_var;
    binary_op op;

    DomainTransitionGraphSubterm(const DomainTransitionGraphSubterm &other); // copying forbidden

public:
    DomainTransitionGraphSubterm(int var_index);
    virtual void read_data(istream &in);
    virtual void dump() const;
    binary_op get_op() const {
	return op;
    }
    int get_left_var() const {
	return left_var;
    }
    int get_right_var() const {
	return right_var;
    }

};

class DomainTransitionGraphFunc : public DomainTransitionGraph {
    friend class CGHeuristic;
    friend class ValueNode;
    friend class ValueTransition;
    friend class Transition;
    friend class LocalProblemNumerical;
    friend class DomainTransitionGraphComp;

    vector<FuncTransition> transitions;

    DomainTransitionGraphFunc(const DomainTransitionGraphFunc &other); // copying forbidden

public:
    DomainTransitionGraphFunc(int var_index);
    virtual void read_data(istream &in);
    virtual void dump() const;
};

class DomainTransitionGraphComp : public DomainTransitionGraph {
    friend class CGHeuristic;
    friend class ValueNode;
    friend class ValueTransition;
    friend class Transition;
    friend class LocalProblemComp;

    //all transitions which modify a primitve functional variable which is a child
    //in the causal graph of this variable
    vector<FuncTransition> transitions;

    DomainTransitionGraphComp(const DomainTransitionGraphComp &other); // copying forbidden
public:
    std::pair<CompTransition, CompTransition> nodes; //first has start value false; second has start value true
    DomainTransitionGraphComp(int var_index);
    virtual void read_data(istream &in);
    void compute_recursively_parents(int var, map<int, int> &global_to_ccg_parent);
    void collect_recursively_func_transitions(int var, map<int, int> &global_to_ccg_parent);
    int translate_global_to_local(map<int, int> &global_to_ccg_parent, int global_var);
    virtual void dump() const;
};

#endif

==> tfd-src-0.2.2.tgz: tfd-src-0.2.2/search/domain_transition_graph.h <==
#ifndef DOMAIN_TRANSITION_GRAPH_H
#define DOMAIN_TRANSITION_GRAPH_H

#include <iostream>
#include <map>
#include <vector>
#include <tr1/unordered_map>
//#include "globals.h"
#include "operator.h"
using namespace std;

class CGHeuristic;
class TimeStampedState;
class Operator;

class ValueNode;
class ValueTransition;
class ValueTransitionLabel;
class DomainTransitionGraph;

// Note: We do not use references but pointers to refer to the "parents" of
// transitions and nodes. This is because these structures could not be
// put into vectors otherwise.

typedef multimap<int, ValueNode *> Heap;
typedef std::tr1::unordered_map<int, int> hashmap;
typedef hashmap::value_type ValuePair;

struct LocalAssignment {
    DomainTransitionGraph *prev_dtg;
    int local_var;
    double value;
    int var;
    assignment_op fop;
    condition_type cond_type;
    LocalAssignment(DomainTransitionGraph *_prev_dtg, int _local_var, double _value, condition_type _cond_type) :
	prev_dtg(_prev_dtg), local_var(_local_var), value(_value), cond_type(_cond_type) {
    	var = -1;
    }
    LocalAssignment(DomainTransitionGraph *_prev_dtg, int _local_var, int _var, assignment_op _fop, condition_type _cond_type) :
	prev_dtg(_prev_dtg), local_var(_local_var), var(_var), fop(_fop), cond_type(_cond_type) {
    	value = -1.0;
    }
    LocalAssignment(istream &in);
    void dump() const;
};
class ValueTransitionLabel {
public:
    int duration_variable;
    vector<LocalAssignment> precond;
    trans_type type;
    vector<LocalAssignment> effect;
    Operator *op;
    ValueTransitionLabel(int the_duration_variable, const vector<LocalAssignment> &the_precond,
            trans_type the_type, const vector<LocalAssignment> &the_effect = vector<LocalAssignment> (),Operator* theOp = NULL) :
        duration_variable(the_duration_variable), precond(the_precond), type(the_type), effect(
                the_effect), op(theOp) {
    }
    virtual ~ValueTransitionLabel() {}
};

class ValueTransition {
public:
    ValueNode *target;
    vector<ValueTransitionLabel*> ccg_labels; // labels for cyclic CG heuristic

    ValueTransition(ValueNode *targ) : target(targ) {}
    void dump() const;
};

struct ValueNode {
    DomainTransitionGraph *parent_graph;
    int value;
    vector<ValueTransition> transitions;
    vector<ValueTransition> additional_transitions;

    ValueNode(DomainTransitionGraph *parent, int val) :
	parent_graph(parent), value(val) {
    }
    void dump() const;
};

struct CompTransition {
    int left_var;
    int right_var;
    binary_op op;

    void dump() const;
};

class FuncTransitionLabel : public ValueTransitionLabel {
public:
    int starting_variable;
    assignment_op a_op;
    int influencing_variable;
    FuncTransitionLabel(int the_starting_variable,
	    vector<LocalAssignment> the_precond, vector<LocalAssignment> the_effect, assignment_op the_a_op,
	    int the_influencing_variable, int the_duration_variable, trans_type type,
	    Operator *the_op) :
		ValueTransitionLabel(the_duration_variable, the_precond, type, the_effect, the_op),
	starting_variable(the_starting_variable), a_op(the_a_op), influencing_variable(the_influencing_variable) {
    }
};

class DomainTransitionGraph {
public:
    int var;
    bool is_axiom;

    vector<int> ccg_parents;
    // used for mapping variables in conditions to their global index
    // (only needed for initializing child_state for the start node?)
    hashmap global_to_local_ccg_parents;

    virtual ~DomainTransitionGraph() {}
    static void read_all(istream &in);
    virtual void read_data(istream &in) = 0;
    static void compute_causal_graph_parents_comp(int var, map<int, int> &global_to_ccg_parent);
    static void collect_func_transitions(int var, map<int, int> &global_to_ccg_parent);
    virtual void dump() const = 0;
protected:
    int translate_global_to_local(map<int, int> &global_to_ccg_parent, int global_var);
};

class DomainTransitionGraphSymb : public DomainTransitionGraph {
public:
    vector<ValueNode> nodes;

    DomainTransitionGraphSymb(int var_index, int node_count);
    virtual void read_data(istream &in);
    virtual void dump() const;
    virtual void get_successors(int value, vector<int> &result) const;
    // Build vector of values v' such that there is a transition from value to v'.

private:
    void compress_effects(const Operator* op, vector<PrePost>& pre_post);
    bool add_relevant_functional_vars_to_context(int var_no, map<int, int> &global_to_ccg_parent);
    void extend_cyclic_effect(const PrePost& pre_post, vector<LocalAssignment>& cyclic_effect,
	map<int, int> &global_to_ccg_parent, const vector<pair<int, int> >& precond_pairs);
    DomainTransitionGraphSymb(const DomainTransitionGraphSymb &other); // copying forbidden
};

class DomainTransitionGraphSubterm : public DomainTransitionGraph {
public:
    int left_var;
    int right_var;
    binary_op op;

    DomainTransitionGraphSubterm(int var_index);
    virtual void read_data(istream &in);
    virtual void dump() const;

private:
    DomainTransitionGraphSubterm(const DomainTransitionGraphSubterm &other); // copying forbidden
};

class DomainTransitionGraphFunc : public DomainTransitionGraph {
public:
    vector<FuncTransitionLabel> transitions;
    DomainTransitionGraphFunc(int var_index);
    virtual void read_data(istream &in);
    virtual void dump() const;

private:
    DomainTransitionGraphFunc(const DomainTransitionGraphFunc &other); // copying forbidden
};

class DomainTransitionGraphComp : public DomainTransitionGraph {
public:
    //all transitions which modify a primitve functional variable which is a child
    //in the causal graph of this variable
	// in contrast to DomainTransitionGraphFunc, here the variables are local
    vector<FuncTransitionLabel> transitions;

    std::pair<CompTransition, CompTransition> nodes; //first has start value false; second has start value true
    DomainTransitionGraphComp(int var_index);
    virtual void read_data(istream &in);
    void compute_recursively_parents(int var, map<int, int> &global_to_ccg_parent);
    void collect_recursively_func_transitions(int var, map<int, int> &global_to_ccg_parent);
    virtual void dump() const;
private:
    DomainTransitionGraphComp(const DomainTransitionGraphComp &other); // copying forbidden
};

#endif
